{
  am_pm_abbreviated => [
    "AM",
    "PM",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E, d.",
    Ehm => "E h:mm\N{U+202f}a",
    Ehms => "E h:mm:ss\N{U+202f}a",
    Gy => "y. G",
    GyMMM => "LLL y. G",
    GyMMMEd => "E, d. MMM y. G",
    GyMMMd => "d. MMM y. G",
    GyMd => "d. M. y. GGGGG",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L.",
    MEd => "E, dd. MM.",
    MMM => "LLL",
    MMMEd => "E, d. MMM",
    MMMMEd => "E, d. MMMM",
    "MMMMW-count-few" => "W. 'tjedan' MMMM",
    "MMMMW-count-one" => "W. 'tjedan' MMMM",
    "MMMMW-count-other" => "W. 'tjedan' MMMM",
    MMMMd => "d. MMMM",
    MMMd => "d. MMM",
    MMdd => "dd. MM.",
    Md => "dd. MM.",
    d => "d.",
    h => "h\N{U+202f}a",
    hm => "hh:mm\N{U+202f}a",
    hms => "hh:mm:ss\N{U+202f}a",
    hmsv => "h:mm:ss\N{U+202f}a v",
    hmv => "h:mm\N{U+202f}a v",
    ms => "mm:ss",
    y => "y.",
    yM => "MM. y.",
    yMEd => "E, dd. MM. y.",
    yMM => "MM. y.",
    yMMM => "LLL y.",
    yMMMEd => "E, d. MMM y.",
    yMMMM => "LLLL y.",
    yMMMd => "d. MMM y.",
    yMd => "dd. MM. y.",
    yQQQ => "QQQ y.",
    yQQQQ => "QQQQ y.",
    "yw-count-few" => "w. 'tjedan' 'u' Y.",
    "yw-count-one" => "w. 'tjedan' 'u' Y.",
    "yw-count-other" => "w. 'tjedan' 'u' Y.",
  },
  code => "hr-BA",
  date_format_full => "EEEE, d. MMMM y.",
  date_format_long => "d. MMMM y.",
  date_format_medium => "d. MMM y.",
  date_format_short => "d. M. yy.",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "pon",
    "uto",
    "sri",
    "\N{U+010d}et",
    "pet",
    "sub",
    "ned",
  ],
  day_format_narrow => [
    "P",
    "U",
    "S",
    "\N{U+010c}",
    "P",
    "S",
    "N",
  ],
  day_format_wide => [
    "ponedjeljak",
    "utorak",
    "srijeda",
    "\N{U+010d}etvrtak",
    "petak",
    "subota",
    "nedjelja",
  ],
  day_stand_alone_abbreviated => [
    "pon",
    "uto",
    "sri",
    "\N{U+010d}et",
    "pet",
    "sub",
    "ned",
  ],
  day_stand_alone_narrow => [
    "P",
    "U",
    "S",
    "\N{U+010c}",
    "P",
    "S",
    "N",
  ],
  day_stand_alone_wide => [
    "ponedjeljak",
    "utorak",
    "srijeda",
    "\N{U+010d}etvrtak",
    "petak",
    "subota",
    "nedjelja",
  ],
  era_abbreviated => [
    "pr. Kr.",
    "po. Kr.",
  ],
  era_narrow => [
    "pr.n.e.",
    "AD",
  ],
  era_wide => [
    "prije Krista",
    "poslije Krista",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Croatian",
  month_format_abbreviated => [
    "sij",
    "velj",
    "o\N{U+017e}u",
    "tra",
    "svi",
    "lip",
    "srp",
    "kol",
    "ruj",
    "lis",
    "stu",
    "pro",
  ],
  month_format_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
    "5.",
    "6.",
    "7.",
    "8.",
    "9.",
    "10.",
    "11.",
    "12.",
  ],
  month_format_wide => [
    "sije\N{U+010d}nja",
    "velja\N{U+010d}e",
    "o\N{U+017e}ujka",
    "travnja",
    "svibnja",
    "lipnja",
    "srpnja",
    "kolovoza",
    "rujna",
    "listopada",
    "studenoga",
    "prosinca",
  ],
  month_stand_alone_abbreviated => [
    "sij",
    "velj",
    "o\N{U+017e}u",
    "tra",
    "svi",
    "lip",
    "srp",
    "kol",
    "ruj",
    "lis",
    "stu",
    "pro",
  ],
  month_stand_alone_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
    "5.",
    "6.",
    "7.",
    "8.",
    "9.",
    "10.",
    "11.",
    "12.",
  ],
  month_stand_alone_wide => [
    "sije\N{U+010d}anj",
    "velja\N{U+010d}a",
    "o\N{U+017e}ujak",
    "travanj",
    "svibanj",
    "lipanj",
    "srpanj",
    "kolovoz",
    "rujan",
    "listopad",
    "studeni",
    "prosinac",
  ],
  name => "Croatian Bosnia & Herzegovina",
  native_language => "hrvatski",
  native_name => "hrvatski Bosna i Hercegovina",
  native_script => undef,
  native_territory => "Bosna i Hercegovina",
  native_variant => undef,
  quarter_format_abbreviated => [
    "1. kv.",
    "2. kv.",
    "3. kv.",
    "4. kv.",
  ],
  quarter_format_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
  ],
  quarter_format_wide => [
    "1. kvartal",
    "2. kvartal",
    "3. kvartal",
    "4. kvartal",
  ],
  quarter_stand_alone_abbreviated => [
    "1. kv.",
    "2. kv.",
    "3. kv.",
    "4. kv.",
  ],
  quarter_stand_alone_narrow => [
    "1.",
    "2.",
    "3.",
    "4.",
  ],
  quarter_stand_alone_wide => [
    "1. kvartal",
    "2. kvartal",
    "3. kvartal",
    "4. kvartal",
  ],
  script => undef,
  territory => "Bosnia & Herzegovina",
  time_format_full => "HH:mm:ss (zzzz)",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
