{
  am_pm_abbreviated => [
    "vm.",
    "nm.",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "hh:mm B",
    Bhms => "hh:mm:ss B",
    E => "ccc",
    EBhm => "E hh:mm B",
    EBhms => "E hh:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E d",
    Ehm => "E hh:mm\N{U+202f}a",
    Ehms => "E hh:mm:ss\N{U+202f}a",
    Gy => "y G",
    GyMMM => "MMM y G",
    GyMMMEd => "E dd MMM y G",
    GyMMMd => "dd MMM y G",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E d/M",
    MMM => "LLL",
    MMMEd => "E d MMM",
    MMMMEd => "E d MMMM",
    "MMMMW-count-one" => "'week' W 'van' MMMM",
    "MMMMW-count-other" => "'week' W 'van' MMMM",
    MMMMd => "d MMMM",
    MMMd => "d MMM",
    Md => "dd-MM",
    d => "d",
    h => "h\N{U+202f}a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss\N{U+202f}a",
    hmsv => "h:mm:ss\N{U+202f}a v",
    hmv => "h:mm\N{U+202f}a v",
    ms => "mm:ss",
    y => "y",
    yM => "MM-y",
    yMEd => "E y-MM-dd",
    yMMM => "MMM y",
    yMMMEd => "E d MMM y",
    yMMMM => "MMMM y",
    yMMMd => "d MMM y",
    yMd => "y-MM-dd",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-one" => "'week' w 'van' Y",
    "yw-count-other" => "'week' w 'van' Y",
  },
  code => "af-NA",
  date_format_full => "EEEE dd MMMM y",
  date_format_long => "dd MMMM y",
  date_format_medium => "dd MMM y",
  date_format_short => "y-MM-dd",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "Ma.",
    "Di.",
    "Wo.",
    "Do.",
    "Vr.",
    "Sa.",
    "So.",
  ],
  day_format_narrow => [
    "M",
    "D",
    "W",
    "D",
    "V",
    "S",
    "S",
  ],
  day_format_wide => [
    "Maandag",
    "Dinsdag",
    "Woensdag",
    "Donderdag",
    "Vrydag",
    "Saterdag",
    "Sondag",
  ],
  day_stand_alone_abbreviated => [
    "Ma.",
    "Di.",
    "Wo.",
    "Do.",
    "Vr.",
    "Sa.",
    "So.",
  ],
  day_stand_alone_narrow => [
    "M",
    "D",
    "W",
    "D",
    "V",
    "S",
    "S",
  ],
  day_stand_alone_wide => [
    "Maandag",
    "Dinsdag",
    "Woensdag",
    "Donderdag",
    "Vrydag",
    "Saterdag",
    "Sondag",
  ],
  era_abbreviated => [
    "v.C.",
    "n.C.",
  ],
  era_narrow => [
    "v.C.",
    "n.C.",
  ],
  era_wide => [
    "voor Christus",
    "na Christus",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Afrikaans",
  month_format_abbreviated => [
    "Jan.",
    "Feb.",
    "Mrt.",
    "Apr.",
    "Mei",
    "Jun.",
    "Jul.",
    "Aug.",
    "Sep.",
    "Okt.",
    "Nov.",
    "Des.",
  ],
  month_format_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_format_wide => [
    "Januarie",
    "Februarie",
    "Maart",
    "April",
    "Mei",
    "Junie",
    "Julie",
    "Augustus",
    "September",
    "Oktober",
    "November",
    "Desember",
  ],
  month_stand_alone_abbreviated => [
    "Jan.",
    "Feb.",
    "Mrt.",
    "Apr.",
    "Mei",
    "Jun.",
    "Jul.",
    "Aug.",
    "Sep.",
    "Okt.",
    "Nov.",
    "Des.",
  ],
  month_stand_alone_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_stand_alone_wide => [
    "Januarie",
    "Februarie",
    "Maart",
    "April",
    "Mei",
    "Junie",
    "Julie",
    "Augustus",
    "September",
    "Oktober",
    "November",
    "Desember",
  ],
  name => "Afrikaans Namibia",
  native_language => "Afrikaans",
  native_name => "Afrikaans Namibi\N{U+00eb}",
  native_script => undef,
  native_territory => "Namibi\N{U+00eb}",
  native_variant => undef,
  quarter_format_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "1ste kwartaal",
    "2de kwartaal",
    "3de kwartaal",
    "4de kwartaal",
  ],
  quarter_stand_alone_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "1ste kwartaal",
    "2de kwartaal",
    "3de kwartaal",
    "4de kwartaal",
  ],
  script => undef,
  territory => "Namibia",
  time_format_full => "h:mm:ss\N{U+202f}a zzzz",
  time_format_long => "h:mm:ss\N{U+202f}a z",
  time_format_medium => "h:mm:ss\N{U+202f}a",
  time_format_short => "h:mm\N{U+202f}a",
  variant => undef,
  version => 43,
}
