{
  am_pm_abbreviated => [
    "pamilau",
    "pamunyi",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "d, E",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E, M/d",
    MMM => "LLL",
    MMMEd => "E, MMM d",
    MMMMEd => "E, MMMM d",
    "MMMMW-count-other" => "'week' W 'of' MMMM",
    MMMMd => "MMMM d",
    MMMd => "MMM d",
    Md => "M/d",
    d => "d",
    h => "h a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E, M/d/y",
    yMMM => "MMM y",
    yMMMEd => "E, MMM d, y",
    yMMMM => "MMMM y",
    yMMMd => "d MMM y",
    yMd => "d/M/y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-other" => "'week' w 'of' Y",
  },
  code => "bez-TZ",
  date_format_full => "EEEE, d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "dd/MM/y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "Vil",
    "Hiv",
    "Hid",
    "Hit",
    "Hih",
    "Lem",
    "Mul",
  ],
  day_format_narrow => [
    "J",
    "H",
    "H",
    "H",
    "W",
    "J",
    "M",
  ],
  day_format_wide => [
    "pa shahuviluha",
    "pa hivili",
    "pa hidatu",
    "pa hitayi",
    "pa hihanu",
    "pa shahulembela",
    "pa mulungu",
  ],
  day_stand_alone_abbreviated => [
    "Vil",
    "Hiv",
    "Hid",
    "Hit",
    "Hih",
    "Lem",
    "Mul",
  ],
  day_stand_alone_narrow => [
    "J",
    "H",
    "H",
    "H",
    "W",
    "J",
    "M",
  ],
  day_stand_alone_wide => [
    "pa shahuviluha",
    "pa hivili",
    "pa hidatu",
    "pa hitayi",
    "pa hihanu",
    "pa shahulembela",
    "pa mulungu",
  ],
  era_abbreviated => [
    "KM",
    "BM",
  ],
  era_narrow => [
    "KM",
    "BM",
  ],
  era_wide => [
    "Kabla ya Mtwaa",
    "Baada ya Mtwaa",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Bena",
  month_format_abbreviated => [
    "Hut",
    "Vil",
    "Dat",
    "Tai",
    "Han",
    "Sit",
    "Sab",
    "Nan",
    "Tis",
    "Kum",
    "Kmj",
    "Kmb",
  ],
  month_format_narrow => [
    "H",
    "V",
    "D",
    "T",
    "H",
    "S",
    "S",
    "N",
    "T",
    "K",
    "K",
    "K",
  ],
  month_format_wide => [
    "pa mwedzi gwa hutala",
    "pa mwedzi gwa wuvili",
    "pa mwedzi gwa wudatu",
    "pa mwedzi gwa wutai",
    "pa mwedzi gwa wuhanu",
    "pa mwedzi gwa sita",
    "pa mwedzi gwa saba",
    "pa mwedzi gwa nane",
    "pa mwedzi gwa tisa",
    "pa mwedzi gwa kumi",
    "pa mwedzi gwa kumi na moja",
    "pa mwedzi gwa kumi na mbili",
  ],
  month_stand_alone_abbreviated => [
    "Hut",
    "Vil",
    "Dat",
    "Tai",
    "Han",
    "Sit",
    "Sab",
    "Nan",
    "Tis",
    "Kum",
    "Kmj",
    "Kmb",
  ],
  month_stand_alone_narrow => [
    "H",
    "V",
    "D",
    "T",
    "H",
    "S",
    "S",
    "N",
    "T",
    "K",
    "K",
    "K",
  ],
  month_stand_alone_wide => [
    "pa mwedzi gwa hutala",
    "pa mwedzi gwa wuvili",
    "pa mwedzi gwa wudatu",
    "pa mwedzi gwa wutai",
    "pa mwedzi gwa wuhanu",
    "pa mwedzi gwa sita",
    "pa mwedzi gwa saba",
    "pa mwedzi gwa nane",
    "pa mwedzi gwa tisa",
    "pa mwedzi gwa kumi",
    "pa mwedzi gwa kumi na moja",
    "pa mwedzi gwa kumi na mbili",
  ],
  name => "Bena Tanzania",
  native_language => "Hibena",
  native_name => "Hibena Hutanzania",
  native_script => undef,
  native_territory => "Hutanzania",
  native_variant => undef,
  quarter_format_abbreviated => [
    "L1",
    "L2",
    "L3",
    "L4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "Lobo 1",
    "Lobo 2",
    "Lobo 3",
    "Lobo 4",
  ],
  quarter_stand_alone_abbreviated => [
    "L1",
    "L2",
    "L3",
    "L4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "Lobo 1",
    "Lobo 2",
    "Lobo 3",
    "Lobo 4",
  ],
  script => undef,
  territory => "Tanzania",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
