h1 {
	font-size: 1.5em;
	font-weight: bold;
}

#something {
	text-decoration: underline;
	font-style: italic;
}

img {
	width: auto;
	height: auto;
	border: 0;
}

a {
	outline: 0;
}

footer {
	display: none;
}
