package Role::Identifiable::HasIdent 0.009;
use Moose::Role;
# ABSTRACT: a thing with an ident attribute

#pod =head1 DESCRIPTION
#pod
#pod This is an incredibly simple role.  It adds a required C<ident> attribute that
#pod stores a simple string, meant to identify exceptions.
#pod
#pod The string has to contain at least one character, and it can't start or end
#pod with whitespace.
#pod
#pod =cut

use Moose::Util::TypeConstraints;

has ident => (
  is  => 'ro',
  isa => subtype('Str', where { length && /\A\S/ && /\S\z/ }),
  required => 1,
);

no Moose::Role;
use Moose::Util::TypeConstraints;
1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Role::Identifiable::HasIdent - a thing with an ident attribute

=head1 VERSION

version 0.009

=head1 DESCRIPTION

This is an incredibly simple role.  It adds a required C<ident> attribute that
stores a simple string, meant to identify exceptions.

The string has to contain at least one character, and it can't start or end
with whitespace.

=head1 PERL VERSION

This library should run on perls released even a long time ago.  It should work
on any version of perl released in the last five years.

Although it may work on older versions of perl, no guarantee is made that the
minimum required version will not be increased.  The version may be increased
for any reason, and there is no promise that patches will be accepted to lower
the minimum required perl.

=head1 AUTHOR

Ricardo Signes <cpan@semiotic.systems>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2022 by Ricardo Signes.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
