{
  am_pm_abbreviated => [
    "FM",
    "FI",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "d E",
    Ehm => "E h:mm\N{U+202f}a",
    Ehms => "E h:mm:ss\N{U+202f}a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E, d/M",
    MMM => "LLL",
    MMMEd => "E, d MMM",
    "MMMMW-count-one" => "'Wik' W 'f\N{U+1ecd}' MMMM",
    "MMMMW-count-other" => "'Wik' W 'f\N{U+1ecd}' MMMM",
    MMMMd => "d MMMM",
    MMMd => "d MMM",
    Md => "d /M",
    d => "d",
    h => "h\N{U+202f}a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss\N{U+202f}a",
    hmsv => "h:mm:ss\N{U+202f}a v",
    hmv => "h:mm\N{U+202f}a v",
    ms => "mm:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E, d/M/y",
    yMMM => "MMM y",
    yMMMEd => "E, d MMM y",
    yMMMM => "MMMM y",
    yMMMd => "d MMM y",
    yMd => "d/M/y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-one" => "'Wik' w 'f\N{U+1ecd}' Y",
    "yw-count-other" => "'Wiik' w 'f\N{U+1ecd}' Y",
  },
  code => "pcm-NG",
  date_format_full => "EEEE, d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "dd/MM/y",
  datetime_format_full => "{1}, {0}",
  datetime_format_long => "{1}, {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "M\N{U+1ecd}\N{U+0301}n",
    "Ti\N{U+00fa}",
    "W\N{U+1eb9}\N{U+0301}n",
    "T\N{U+1ecd}\N{U+0301}z",
    "Fra\N{U+00ed}",
    "S\N{U+00e1}t",
    "S\N{U+1ecd}\N{U+0301}n",
  ],
  day_format_narrow => [
    "M",
    "T",
    "W",
    "T",
    "F",
    "S",
    "S",
  ],
  day_format_wide => [
    "M\N{U+1ecd}\N{U+0301}nd\N{U+00e8}",
    "Ti\N{U+00fa}zd\N{U+00e8}",
    "W\N{U+1eb9}\N{U+0301}n\N{U+1eb9}\N{U+0301}zd\N{U+00e8}",
    "T\N{U+1ecd}\N{U+0301}zd\N{U+00e8}",
    "Fra\N{U+00ed}d\N{U+00e8}",
    "S\N{U+00e1}t\N{U+1ecd}d\N{U+00e8}",
    "S\N{U+1ecd}\N{U+0301}nd\N{U+00e8}",
  ],
  day_stand_alone_abbreviated => [
    "M\N{U+1ecd}\N{U+0301}n",
    "Ti\N{U+00fa}",
    "W\N{U+1eb9}\N{U+0301}n",
    "T\N{U+1ecd}\N{U+0301}z",
    "Fra\N{U+00ed}",
    "S\N{U+00e1}t",
    "S\N{U+1ecd}\N{U+0301}n",
  ],
  day_stand_alone_narrow => [
    "M",
    "T",
    "W",
    "T",
    "F",
    "S",
    "S",
  ],
  day_stand_alone_wide => [
    "M\N{U+1ecd}\N{U+0301}nd\N{U+00e8}",
    "Ti\N{U+00fa}zd\N{U+00e8}",
    "W\N{U+1eb9}\N{U+0301}n\N{U+1eb9}\N{U+0301}zd\N{U+00e8}",
    "T\N{U+1ecd}\N{U+0301}zd\N{U+00e8}",
    "Fra\N{U+00ed}d\N{U+00e8}",
    "S\N{U+00e1}t\N{U+1ecd}d\N{U+00e8}",
    "S\N{U+1ecd}\N{U+0301}nd\N{U+00e8}",
  ],
  era_abbreviated => [
    "BK",
    "KIY",
  ],
  era_narrow => [
    "BK",
    "KIY",
  ],
  era_wide => [
    "Bif\N{U+1ecd}\N{U+0301} Kraist",
    "Kraist Im Yi\N{U+1eb9}",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Nigerian Pidgin",
  month_format_abbreviated => [
    "J\N{U+00e9}n",
    "F\N{U+1eb9}\N{U+0301}b",
    "Mach",
    "\N{U+00c9}pr",
    "Mee",
    "Jun",
    "Jul",
    "\N{U+1ecc}g\N{U+1ecd}",
    "S\N{U+1eb9}p",
    "\N{U+1ecc}kt",
    "N\N{U+1ecd}v",
    "Dis",
  ],
  month_format_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_format_wide => [
    "J\N{U+00e9}n\N{U+00fa}\N{U+00e1}ri",
    "F\N{U+1eb9}\N{U+0301}b\N{U+00fa}\N{U+00e1}ri",
    "Mach",
    "\N{U+00c9}prel",
    "Mee",
    "Jun",
    "Julai",
    "\N{U+1ecc}g\N{U+1ecd}st",
    "S\N{U+1eb9}pt\N{U+1eb9}\N{U+0301}mba",
    "\N{U+1ecc}kt\N{U+00f3}ba",
    "N\N{U+1ecd}v\N{U+1eb9}\N{U+0301}mba",
    "Dis\N{U+1eb9}\N{U+0301}mba",
  ],
  month_stand_alone_abbreviated => [
    "J\N{U+00e9}n",
    "F\N{U+1eb9}\N{U+0301}b",
    "Mach",
    "\N{U+00c9}pr",
    "Mee",
    "Jun",
    "Jul",
    "\N{U+1ecc}\N{U+0301}g\N{U+1ecd}",
    "S\N{U+1eb9}p",
    "\N{U+1ecc}kt",
    "N\N{U+1ecd}v",
    "Dis",
  ],
  month_stand_alone_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "A",
    "S",
    "O",
    "N",
    "D",
  ],
  month_stand_alone_wide => [
    "J\N{U+00e9}n\N{U+00fa}\N{U+00e1}ri",
    "F\N{U+1eb9}\N{U+0301}b\N{U+00fa}\N{U+00e1}ri",
    "Mach",
    "\N{U+00c9}prel",
    "Mee",
    "Jun",
    "Julai",
    "\N{U+1ecc}g\N{U+1ecd}st",
    "S\N{U+1eb9}pt\N{U+1eb9}\N{U+0301}mba",
    "\N{U+1ecc}kt\N{U+00f3}ba",
    "N\N{U+1ecd}v\N{U+1eb9}\N{U+0301}mba",
    "Dis\N{U+1eb9}\N{U+0301}mba",
  ],
  name => "Nigerian Pidgin Nigeria",
  native_language => "Naij\N{U+00ed}ri\N{U+00e1} P\N{U+00ed}jin",
  native_name => "Naij\N{U+00ed}ri\N{U+00e1} P\N{U+00ed}jin Naij\N{U+00ed}ria",
  native_script => undef,
  native_territory => "Naij\N{U+00ed}ria",
  native_variant => undef,
  quarter_format_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "F\N{U+1eb9}\N{U+0301}st Kw\N{U+1ecd}\N{U+0301}ta",
    "S\N{U+1eb9}\N{U+0301}k\N{U+1ecd}\N{U+0301}n Kw\N{U+1ecd}\N{U+0301}ta",
    "T\N{U+1ecd}\N{U+0301}d Kw\N{U+1ecd}\N{U+0301}ta",
    "F\N{U+1ecd}\N{U+0301}t Kw\N{U+1ecd}\N{U+0301}ta",
  ],
  quarter_stand_alone_abbreviated => [
    "K1",
    "K2",
    "K3",
    "K4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "F\N{U+1eb9}\N{U+0301}st Kw\N{U+1ecd}\N{U+0301}ta",
    "S\N{U+1eb9}\N{U+0301}k\N{U+1ecd}\N{U+0301}n Kw\N{U+1ecd}\N{U+0301}ta",
    "T\N{U+1ecd}\N{U+0301}d Kw\N{U+1ecd}\N{U+0301}ta",
    "F\N{U+1ecd}\N{U+0301}t Kw\N{U+1ecd}\N{U+0301}ta",
  ],
  script => undef,
  territory => "Nigeria",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "H:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
