{
  am_pm_abbreviated => [
    "\N{U+0128}yakwakya",
    "\N{U+0128}yaw\N{U+0129}oo",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "d, E",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E, M/d",
    MMM => "LLL",
    MMMEd => "E, MMM d",
    MMMMEd => "E, MMMM d",
    "MMMMW-count-other" => "'week' W 'of' MMMM",
    MMMMd => "MMMM d",
    MMMd => "MMM d",
    Md => "M/d",
    d => "d",
    h => "h a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E, M/d/y",
    yMMM => "MMM y",
    yMMMEd => "E, MMM d, y",
    yMMMM => "MMMM y",
    yMMMd => "y MMM d",
    yMd => "y-MM-dd",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-other" => "'week' w 'of' Y",
  },
  code => "kam",
  date_format_full => "EEEE, d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "dd/MM/y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "Wkw",
    "Wkl",
    "Wt\N{U+0169}",
    "Wkn",
    "Wtn",
    "Wth",
    "Wky",
  ],
  day_format_narrow => [
    "W",
    "E",
    "A",
    "A",
    "A",
    "A",
    "Y",
  ],
  day_format_wide => [
    "Wa kwamb\N{U+0129}l\N{U+0129}lya",
    "Wa kel\N{U+0129}",
    "Wa katat\N{U+0169}",
    "Wa kana",
    "Wa katano",
    "Wa thanthat\N{U+0169}",
    "Wa kyumwa",
  ],
  day_stand_alone_abbreviated => [
    "Wkw",
    "Wkl",
    "Wt\N{U+0169}",
    "Wkn",
    "Wtn",
    "Wth",
    "Wky",
  ],
  day_stand_alone_narrow => [
    "W",
    "E",
    "A",
    "A",
    "A",
    "A",
    "Y",
  ],
  day_stand_alone_wide => [
    "Wa kwamb\N{U+0129}l\N{U+0129}lya",
    "Wa kel\N{U+0129}",
    "Wa katat\N{U+0169}",
    "Wa kana",
    "Wa katano",
    "Wa thanthat\N{U+0169}",
    "Wa kyumwa",
  ],
  era_abbreviated => [
    "MY",
    "IY",
  ],
  era_narrow => [
    "MY",
    "IY",
  ],
  era_wide => [
    "Mbee wa Yes\N{U+0169}",
    "\N{U+0128}tina wa Yes\N{U+0169}",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Kamba",
  month_format_abbreviated => [
    "Mbe",
    "Kel",
    "Kt\N{U+0169}",
    "Kan",
    "Ktn",
    "Tha",
    "Moo",
    "Nya",
    "Knd",
    "\N{U+0128}ku",
    "\N{U+0128}km",
    "\N{U+0128}kl",
  ],
  month_format_narrow => [
    "M",
    "K",
    "K",
    "K",
    "K",
    "T",
    "M",
    "N",
    "K",
    "\N{U+0128}",
    "\N{U+0128}",
    "\N{U+0128}",
  ],
  month_format_wide => [
    "Mwai wa mbee",
    "Mwai wa kel\N{U+0129}",
    "Mwai wa katat\N{U+0169}",
    "Mwai wa kana",
    "Mwai wa katano",
    "Mwai wa thanthat\N{U+0169}",
    "Mwai wa muonza",
    "Mwai wa nyaanya",
    "Mwai wa kenda",
    "Mwai wa \N{U+0129}kumi",
    "Mwai wa \N{U+0129}kumi na \N{U+0129}mwe",
    "Mwai wa \N{U+0129}kumi na il\N{U+0129}",
  ],
  month_stand_alone_abbreviated => [
    "Mbe",
    "Kel",
    "Kt\N{U+0169}",
    "Kan",
    "Ktn",
    "Tha",
    "Moo",
    "Nya",
    "Knd",
    "\N{U+0128}ku",
    "\N{U+0128}km",
    "\N{U+0128}kl",
  ],
  month_stand_alone_narrow => [
    "M",
    "K",
    "K",
    "K",
    "K",
    "T",
    "M",
    "N",
    "K",
    "\N{U+0128}",
    "\N{U+0128}",
    "\N{U+0128}",
  ],
  month_stand_alone_wide => [
    "Mwai wa mbee",
    "Mwai wa kel\N{U+0129}",
    "Mwai wa katat\N{U+0169}",
    "Mwai wa kana",
    "Mwai wa katano",
    "Mwai wa thanthat\N{U+0169}",
    "Mwai wa muonza",
    "Mwai wa nyaanya",
    "Mwai wa kenda",
    "Mwai wa \N{U+0129}kumi",
    "Mwai wa \N{U+0129}kumi na \N{U+0129}mwe",
    "Mwai wa \N{U+0129}kumi na il\N{U+0129}",
  ],
  name => "Kamba",
  native_language => "Kikamba",
  native_name => "Kikamba",
  native_script => undef,
  native_territory => undef,
  native_variant => undef,
  quarter_format_abbreviated => [
    "L1",
    "L2",
    "L3",
    "L4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "Lovo ya mbee",
    "Lovo ya kel\N{U+0129}",
    "Lovo ya katat\N{U+0169}",
    "Lovo ya kana",
  ],
  quarter_stand_alone_abbreviated => [
    "L1",
    "L2",
    "L3",
    "L4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "Lovo ya mbee",
    "Lovo ya kel\N{U+0129}",
    "Lovo ya katat\N{U+0169}",
    "Lovo ya kana",
  ],
  script => undef,
  territory => undef,
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
