{
  am_pm_abbreviated => [
    "v.M.",
    "n.M.",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E 'd\N{U+00e4}' d.",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "H:mm",
    Hms => "H:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E d. M.",
    MMM => "LLL",
    MMMEd => "E d. MMM",
    "MMMMW-count-other" => "'week' W 'of' MMMM",
    MMMMd => "d. MMMM",
    MMMd => "d. MMM",
    Md => "MM-dd",
    d => "d",
    h => "h a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss\N{U+202f}a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "mm:ss",
    y => "y",
    yM => "Y-MM",
    yMEd => "E y-MM-dd",
    yMMM => "MMM y",
    yMMMEd => "E d. MMM. y",
    yMMMM => "MMMM y",
    yMMMd => "d. MMM. y",
    yMd => "y-MM-dd",
    yQQQ => "QQQy",
    yQQQQ => "QQQQ y",
    "yw-count-other" => "'week' w 'of' Y",
  },
  code => "ksh-DE",
  date_format_full => "EEEE, 'd\N{U+00e4}' d. MMMM y",
  date_format_long => "d. MMMM y",
  date_format_medium => "d. MMM. y",
  date_format_short => "d. M. y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "Mo.",
    "Di.",
    "Me.",
    "Du.",
    "Fr.",
    "Sa.",
    "Su.",
  ],
  day_format_narrow => [
    "M",
    "D",
    "M",
    "D",
    "F",
    "S",
    "S",
  ],
  day_format_wide => [
    "Mohndaach",
    "Dinnsdaach",
    "Metwoch",
    "Dunnersdaach",
    "Friidaach",
    "Samsdaach",
    "Sunndaach",
  ],
  day_stand_alone_abbreviated => [
    "Mo.",
    "Di.",
    "Me.",
    "Du.",
    "Fr.",
    "Sa.",
    "Su.",
  ],
  day_stand_alone_narrow => [
    "M",
    "D",
    "M",
    "D",
    "F",
    "S",
    "S",
  ],
  day_stand_alone_wide => [
    "Mohndaach",
    "Dinnsdaach",
    "Metwoch",
    "Dunnersdaach",
    "Friidaach",
    "Samsdaach",
    "Sunndaach",
  ],
  era_abbreviated => [
    "v. Chr.",
    "n. Chr.",
  ],
  era_narrow => [
    "vC",
    "nC",
  ],
  era_wide => [
    "v\N{U+00fc}r Krestos",
    "noh Krestos",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Colognian",
  month_format_abbreviated => [
    "Jan",
    "F\N{U+00e4}b",
    "M\N{U+00e4}z",
    "Apr",
    "Mai",
    "Jun",
    "Jul",
    "Ouj",
    "S\N{U+00e4}p",
    "Okt",
    "Nov",
    "Dez",
  ],
  month_format_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "O",
    "S",
    "O",
    "N",
    "D",
  ],
  month_format_wide => [
    "Jannewa",
    "F\N{U+00e4}browa",
    "M\N{U+00e4}\N{U+00e4}z",
    "Aprell",
    "Mai",
    "Juuni",
    "Juuli",
    "Oujo\N{U+00df}",
    "Sept\N{U+00e4}mber",
    "Oktohber",
    "Nov\N{U+00e4}mber",
    "Dez\N{U+00e4}mber",
  ],
  month_stand_alone_abbreviated => [
    "Jan.",
    "F\N{U+00e4}b.",
    "M\N{U+00e4}z.",
    "Apr.",
    "Mai",
    "Jun.",
    "Jul.",
    "Ouj.",
    "S\N{U+00e4}p.",
    "Okt.",
    "Nov.",
    "Dez.",
  ],
  month_stand_alone_narrow => [
    "J",
    "F",
    "M",
    "A",
    "M",
    "J",
    "J",
    "O",
    "S",
    "O",
    "N",
    "D",
  ],
  month_stand_alone_wide => [
    "Jannewa",
    "F\N{U+00e4}browa",
    "M\N{U+00e4}\N{U+00e4}z",
    "Aprell",
    "Mai",
    "Juuni",
    "Juuli",
    "Oujo\N{U+00df}",
    "Sept\N{U+00e4}mber",
    "Oktohber",
    "Nov\N{U+00e4}mber",
    "Dez\N{U+00e4}mber",
  ],
  name => "Colognian Germany",
  native_language => "K\N{U+00f6}lsch",
  native_name => "K\N{U+00f6}lsch Do\N{U+00fc}tschland",
  native_script => undef,
  native_territory => "Do\N{U+00fc}tschland",
  native_variant => undef,
  quarter_format_abbreviated => [
    "1.Q.",
    "2.Q.",
    "3.Q.",
    "4.Q.",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "1. Quattahl",
    "2. Quattahl",
    "3. Quattahl",
    "4. Quattahl",
  ],
  quarter_stand_alone_abbreviated => [
    "1.Q.",
    "2.Q.",
    "3.Q.",
    "4.Q.",
  ],
  quarter_stand_alone_narrow => [
    "1Q",
    "2Q",
    "3Q",
    "4Q",
  ],
  quarter_stand_alone_wide => [
    "1. Quattahl",
    "2. Quattahl",
    "3. Quattahl",
    "4. Quattahl",
  ],
  script => undef,
  territory => "Germany",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
