{
  am_pm_abbreviated => [
    "Dinda",
    "Dilolo",
  ],
  available_formats => {
    Bh => "h B",
    Bhm => "h:mm B",
    Bhms => "h:mm:ss B",
    E => "ccc",
    EBhm => "E h:mm B",
    EBhms => "E h:mm:ss B",
    EHm => "E HH:mm",
    EHms => "E HH:mm:ss",
    Ed => "E d",
    Ehm => "E h:mm a",
    Ehms => "E h:mm:ss a",
    Gy => "G y",
    GyMMM => "G y MMM",
    GyMMMEd => "G y MMM d, E",
    GyMMMd => "G y MMM d",
    GyMd => "GGGGG y-MM-dd",
    H => "HH",
    Hm => "HH:mm",
    Hms => "HH:mm:ss",
    Hmsv => "HH:mm:ss v",
    Hmv => "HH:mm v",
    M => "L",
    MEd => "E d/M",
    MMM => "LLL",
    MMMEd => "E d MMM",
    "MMMMW-count-other" => "'week' W 'of' MMMM",
    MMMMd => "MMMM d",
    MMMd => "d MMM",
    Md => "d/M",
    d => "d",
    h => "h a",
    hm => "h:mm\N{U+202f}a",
    hms => "h:mm:ss\N{U+202f}a",
    hmsv => "h:mm:ss a v",
    hmv => "h:mm a v",
    ms => "m:ss",
    y => "y",
    yM => "M/y",
    yMEd => "E d/M/y",
    yMMM => "MMM y",
    yMMMEd => "E d MMM y",
    yMMMM => "y MMMM",
    yMMMd => "d MMM y",
    yMd => "d/M/y",
    yQQQ => "QQQ y",
    yQQQQ => "QQQQ y",
    "yw-count-other" => "'week' w 'of' Y",
  },
  code => "lu-CD",
  date_format_full => "EEEE d MMMM y",
  date_format_long => "d MMMM y",
  date_format_medium => "d MMM y",
  date_format_short => "d/M/y",
  datetime_format_full => "{1} {0}",
  datetime_format_long => "{1} {0}",
  datetime_format_medium => "{1} {0}",
  datetime_format_short => "{1} {0}",
  day_format_abbreviated => [
    "Nko",
    "Ndy",
    "Ndg",
    "Njw",
    "Ngv",
    "Lub",
    "Lum",
  ],
  day_format_narrow => [
    "N",
    "N",
    "N",
    "N",
    "N",
    "L",
    "L",
  ],
  day_format_wide => [
    "Nkodya",
    "Nd\N{U+00e0}ay\N{U+00e0}",
    "Ndang\N{U+00f9}",
    "Nj\N{U+00f2}wa",
    "Ng\N{U+00f2}vya",
    "Lubingu",
    "Lumingu",
  ],
  day_stand_alone_abbreviated => [
    "Nko",
    "Ndy",
    "Ndg",
    "Njw",
    "Ngv",
    "Lub",
    "Lum",
  ],
  day_stand_alone_narrow => [
    "N",
    "N",
    "N",
    "N",
    "N",
    "L",
    "L",
  ],
  day_stand_alone_wide => [
    "Nkodya",
    "Nd\N{U+00e0}ay\N{U+00e0}",
    "Ndang\N{U+00f9}",
    "Nj\N{U+00f2}wa",
    "Ng\N{U+00f2}vya",
    "Lubingu",
    "Lumingu",
  ],
  era_abbreviated => [
    "kmp. Y.K.",
    "kny. Y. K.",
  ],
  era_narrow => [
    "kmp. Y.K.",
    "kny. Y. K.",
  ],
  era_wide => [
    "Kumpala kwa Yezu Kli",
    "Kunyima kwa Yezu Kli",
  ],
  first_day_of_week => 1,
  glibc_date_1_format => "%a %b %e %H:%M:%S %Z %Y",
  glibc_date_format => "%m/%d/%y",
  glibc_datetime_format => "%a %b %e %H:%M:%S %Y",
  glibc_time_12_format => "%I:%M:%S %p",
  glibc_time_format => "%H:%M:%S",
  language => "Luba-Katanga",
  month_format_abbreviated => [
    "Cio",
    "Lui",
    "Lus",
    "Muu",
    "Lum",
    "Luf",
    "Kab",
    "Lush",
    "Lut",
    "Lun",
    "Kas",
    "Cis",
  ],
  month_format_narrow => [
    "C",
    "L",
    "L",
    "M",
    "L",
    "L",
    "K",
    "L",
    "L",
    "L",
    "K",
    "C",
  ],
  month_format_wide => [
    "Ciongo",
    "L\N{U+00f9}ishi",
    "Lus\N{U+00f2}lo",
    "M\N{U+00f9}uy\N{U+00e0}",
    "Lum\N{U+00f9}ng\N{U+00f9}l\N{U+00f9}",
    "Lufuimi",
    "Kab\N{U+00e0}l\N{U+00e0}sh\N{U+00ec}p\N{U+00f9}",
    "L\N{U+00f9}sh\N{U+00ec}k\N{U+00e0}",
    "Lutongolo",
    "Lung\N{U+00f9}di",
    "Kasw\N{U+00e8}k\N{U+00e8}s\N{U+00e8}",
    "Cisw\N{U+00e0}",
  ],
  month_stand_alone_abbreviated => [
    "Cio",
    "Lui",
    "Lus",
    "Muu",
    "Lum",
    "Luf",
    "Kab",
    "Lush",
    "Lut",
    "Lun",
    "Kas",
    "Cis",
  ],
  month_stand_alone_narrow => [
    "C",
    "L",
    "L",
    "M",
    "L",
    "L",
    "K",
    "L",
    "L",
    "L",
    "K",
    "C",
  ],
  month_stand_alone_wide => [
    "Ciongo",
    "L\N{U+00f9}ishi",
    "Lus\N{U+00f2}lo",
    "M\N{U+00f9}uy\N{U+00e0}",
    "Lum\N{U+00f9}ng\N{U+00f9}l\N{U+00f9}",
    "Lufuimi",
    "Kab\N{U+00e0}l\N{U+00e0}sh\N{U+00ec}p\N{U+00f9}",
    "L\N{U+00f9}sh\N{U+00ec}k\N{U+00e0}",
    "Lutongolo",
    "Lung\N{U+00f9}di",
    "Kasw\N{U+00e8}k\N{U+00e8}s\N{U+00e8}",
    "Cisw\N{U+00e0}",
  ],
  name => "Luba-Katanga Congo - Kinshasa",
  native_language => "Tshiluba",
  native_name => "Tshiluba Ditunga wa Kongu",
  native_script => undef,
  native_territory => "Ditunga wa Kongu",
  native_variant => undef,
  quarter_format_abbreviated => [
    "M1",
    "M2",
    "M3",
    "M4",
  ],
  quarter_format_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_format_wide => [
    "Mueji 1",
    "Mueji 2",
    "Mueji 3",
    "Mueji 4",
  ],
  quarter_stand_alone_abbreviated => [
    "M1",
    "M2",
    "M3",
    "M4",
  ],
  quarter_stand_alone_narrow => [
    1,
    2,
    3,
    4,
  ],
  quarter_stand_alone_wide => [
    "Mueji 1",
    "Mueji 2",
    "Mueji 3",
    "Mueji 4",
  ],
  script => undef,
  territory => "Congo - Kinshasa",
  time_format_full => "HH:mm:ss zzzz",
  time_format_long => "HH:mm:ss z",
  time_format_medium => "HH:mm:ss",
  time_format_short => "HH:mm",
  variant => undef,
  version => 43,
}
